"""Base tool class for Anyparser integration."""

from enum import Enum
from typing import List, Optional, Union

from anyparser_core import Anyparser
from anyparser_core.options import AnyparserOption
from anyparser_core.parser import AnyparserResult
from crewai.tools import BaseTool
from pydantic import BaseModel, ConfigDict, Field


class AnyparserFormatEnum(str, Enum):
    """Format options for Anyparser output."""

    JSON = "json"
    MARKDOWN = "markdown"
    HTML = "html"


class AnyparserModelEnum(str, Enum):
    """Model options for Anyparser processing."""

    TEXT = "text"
    OCR = "ocr"
    VLM = "vlm"
    LAM = "lam"
    CRAWLER = "crawler"


class EncodingEnum(str, Enum):
    """Encoding options for Anyparser input."""

    UTF8 = "utf-8"
    LATIN1 = "latin1"


class AnyparserBaseConfig(BaseModel):
    """Configuration for Anyparser tools."""

    api_key: Optional[str] = Field(
        default=None,
        description="Anyparser API key. If not provided, will use ANYPARSER_API_KEY env variable",
    )
    api_url: Optional[str] = Field(
        default=None,
        description="Anyparser API URL. If not provided, will use ANYPARSER_API_URL env variable",
    )
    format: AnyparserFormatEnum = Field(
        default=AnyparserFormatEnum.JSON,
        description="Output format (json, markdown, or html)",
    )
    model: AnyparserModelEnum = Field(
        default=AnyparserModelEnum.TEXT,
        description="Processing model to use",
    )
    encoding: EncodingEnum = Field(
        default=EncodingEnum.UTF8,
        description="Input encoding",
    )
    image: Optional[bool] = Field(
        default=None,
        description="Whether to extract images",
    )
    table: Optional[bool] = Field(
        default=None,
        description="Whether to extract tables",
    )


class AnyparserBaseTool(BaseTool):
    """Base class for all Anyparser tools with common configuration options."""

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        populate_by_name=True,
        use_enum_values=False,
    )

    name: str = Field(description="Name of the tool")
    description: str = Field(description="Description of what the tool does")
    api_key: Optional[str] = Field(
        default=None,
        description="Anyparser API key. If not provided, will use ANYPARSER_API_KEY env variable",
    )
    api_url: Optional[str] = Field(
        default=None,
        description="Anyparser API URL. If not provided, will use ANYPARSER_API_URL env variable",
    )
    format: AnyparserFormatEnum = Field(
        default=AnyparserFormatEnum.JSON,
        description="Output format (json, markdown, or html)",
    )
    model: AnyparserModelEnum = Field(
        default=AnyparserModelEnum.TEXT,
        description="Processing model to use",
    )
    encoding: EncodingEnum = Field(
        default=EncodingEnum.UTF8,
        description="Input encoding",
    )
    image: Optional[bool] = Field(
        default=None,
        description="Whether to extract images",
    )
    table: Optional[bool] = Field(
        default=None,
        description="Whether to extract tables",
    )

    def _get_parser_options(self, **kwargs) -> AnyparserOption:
        """Get Anyparser options with current tool settings.

        Args:
            **kwargs: Additional options to override tool settings.

        Returns:
            AnyparserOption instance with combined settings.
        """
        import os

        # Start with base options
        options = {
            "api_key": self.api_key or os.getenv("ANYPARSER_API_KEY"),
            "api_url": self.api_url or os.getenv("ANYPARSER_API_URL"),
            "format": self.format.value,
            "model": self.model.value,
            "encoding": self.encoding.value,
        }

        # Add optional base settings if set
        if self.image is not None:
            options["image"] = self.image
        if self.table is not None:
            options["table"] = self.table

        # Add any additional options from subclasses
        for key, value in kwargs.items():
            if value is not None:  # Only add non-None values
                options[key] = value

        # Create parser instance with combined options
        parser_options = AnyparserOption(**options)
        return parser_options

    def _get_parser(self) -> Anyparser:
        """Get or create an Anyparser instance."""
        if not hasattr(self, "_parser"):
            options = self._get_parser_options()
            self._parser = Anyparser(options=options)
        return self._parser

    async def _parse_async(
        self, input: Union[str, List[str]]
    ) -> Union[str, List[AnyparserResult]]:
        """Parse input asynchronously."""
        parser = self._get_parser()
        return await parser.parse(input)

    def _parse_sync(
        self, input: Union[str, List[str]]
    ) -> Union[str, List[AnyparserResult]]:
        """Parse input synchronously."""
        import asyncio

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            result = loop.run_until_complete(self._parse_async(input))
            if isinstance(result, list):
                # Join multiple results if needed
                markdowns = [
                    r.markdown
                    for r in result
                    if hasattr(r, "markdown") and r.markdown is not None
                ]
                return "\n\n".join(filter(None, markdowns))  # Filter out empty strings
            return result
        finally:
            loop.close()
