"""Crawler tool for Anyparser integration."""

from enum import Enum

from anyparser_core import Anyparser
from pydantic import ConfigDict, Field

from .base import AnyparserBaseTool, AnyparserModelEnum


class StrategyEnum(str, Enum):
    """Strategy options for web crawling."""

    LIFO = "LIFO"
    FIFO = "FIFO"


class TraversalScopeEnum(str, Enum):
    """Traversal scope options for web crawling."""

    SUBTREE = "subtree"
    DOMAIN = "domain"


class AnyparserCrawlerTool(AnyparserBaseTool):
    """Tool for crawling websites using Anyparser.

    This tool can crawl websites and extract their content with configurable settings
    like crawling depth, strategy, and scope.
    """

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
        populate_by_name=True,
        use_enum_values=False,
    )

    name: str = "anyparserbot"
    description: str = """
    Crawl websites and extract their content. Features:
    - Configurable crawling depth and limits
    - Different crawling strategies (LIFO/FIFO)
    - Scope control (subtree/domain)
    - Respects robots.txt
    
    Provide the URL to crawl.
    """

    model: AnyparserModelEnum = Field(
        default=AnyparserModelEnum.CRAWLER,
        description="Processing model (always crawler for this tool)",
    )
    max_depth: int = Field(
        default=2,
        description="Maximum crawling depth",
        ge=1,
        le=50,
    )
    max_executions: int = Field(
        default=5,
        description="Maximum number of pages to crawl",
        ge=1,
        le=100,
    )
    strategy: StrategyEnum = Field(
        default=StrategyEnum.LIFO,
        description="Crawling strategy (LIFO or FIFO)",
    )
    traversal_scope: TraversalScopeEnum = Field(
        default=TraversalScopeEnum.SUBTREE,
        description="Crawling scope (subtree or domain)",
    )

    def _run(self, url: str) -> str:
        """Crawl a website and extract its content.

        Args:
            url: The URL to crawl.

        Returns:
            The extracted content in the specified format.
        """
        import asyncio
        import os

        try:
            # Initialize parser with crawler options
            if not hasattr(self, "_parser"):
                self._parser = Anyparser(
                    options=self._get_parser_options(
                        max_depth=self.max_depth,
                        max_executions=self.max_executions,
                        strategy=self.strategy.value,
                        traversal_scope=self.traversal_scope.value,
                    )
                )

            # Run the async parse in a new event loop
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            try:
                result = loop.run_until_complete(self._parser.parse(url))
                if isinstance(result, list):
                    return "\n\n".join(
                        [
                            str(r.markdown)
                            for r in result
                            if getattr(r, "markdown", None)
                        ]
                    )
                elif (
                    not isinstance(result, str)
                    and hasattr(result, "markdown")
                    and result.markdown
                ):
                    return result.markdown
                elif isinstance(result, str):
                    return result
                return ""
            finally:
                loop.close()

        except Exception as e:
            return f"Error crawling website: {str(e)}"
