"""Tool for reading files using Anyparser."""

from typing import Optional

from pydantic import Field

from .base import AnyparserBaseTool


class FileReadTool(AnyparserBaseTool):
    """Tool for reading and extracting content from files."""

    name: str = "anyparser_file_read"
    description: str = """
    Read and extract content from files. Can handle:
    - PDF documents (with text, tables, and images)
    - Images (with OCR capabilities)
    - Web pages
    
    Provide the file path to read and extract content from.
    """

    file_path: Optional[str] = Field(
        default=None,
        description="File path that this tool is restricted to read",
    )

    def _run(self, *args) -> str:
        """Read and extract content from a file.

        Args:
            args[0] (optional): Path to the file to read. Must match the initialized path if file_path is set.

        Returns:
            The extracted content in the specified format.
        """
        try:
            if len(args) > 0:
                input_path = args[0]
                # Validate the file path against allowed path if restricted
                if self.file_path is not None and input_path != self.file_path:
                    return f"Error: Can only read the following file: {self.file_path}"
                path = input_path
            else:
                if self.file_path is None:
                    return "Error: No file path provided"
                path = self.file_path

            result = self._parse_sync(path)

            if isinstance(result, str):
                return result
            else:
                return str(result)

        except Exception as e:
            return f"Error reading file: {str(e)}"
