"""Tool for reading multiple files using Anyparser."""

import glob
from typing import List, Optional

from pydantic import Field

from .base import AnyparserBaseTool


class MultiFileReadTool(AnyparserBaseTool):
    """Tool for reading and extracting content from multiple files."""

    name: str = "anyparser_multi_file_read"
    description: str = """
    Read and extract content from multiple files. Can handle:
    - PDF documents (with text, tables, and images)
    - Word documents (.docx)
    - Web pages
    
    Provide either:
    - A list of file paths to read
    - A directory path to read all supported files from
    - A glob pattern to match files
    """

    file_paths: Optional[List[str]] = Field(
        default=None,
        description="List of file paths that this tool is restricted to read",
    )
    max_files: int = Field(
        default=5,
        description="Maximum number of files to read when using directory or glob pattern",
    )

    def _run(self, *args) -> str:
        """Read and extract content from multiple files.

        Args:
            args[0] (optional): Either:
                - A list of file paths to read
                - A directory path to read all supported files from
                - A glob pattern to match files

        Returns:
            The extracted content in the specified format (markdown or json).
        """
        try:
            if len(args) > 0:
                input_val = args[0]
                # Handle different input types
                if isinstance(input_val, str):
                    # Treat as directory or glob pattern
                    paths = glob.glob(input_val)
                    if not paths:
                        return f"Error: No files found matching pattern: {input_val}"
                    paths = paths[: self.max_files]  # Limit number of files
                else:
                    paths = input_val

                # Validate paths if restricted
                if self.file_paths is not None:
                    invalid_paths = [p for p in paths if p not in self.file_paths]
                    if invalid_paths:
                        return f"Error: Can only read the following files: {self.file_paths}"
            else:
                if self.file_paths is None:
                    return "Error: No file paths provided"
                paths = self.file_paths

            result = self._parse_sync(paths)

            if isinstance(result, str):
                return result
            return "Error: Unexpected result type"
        except Exception as e:
            return f"Error reading files: {str(e)}"
