"""Tool for OCR processing using Anyparser."""

from typing import List, Optional

from anyparser_core import Anyparser, OcrLanguage, OCRPreset
from pydantic import Field

from .base import AnyparserBaseTool, AnyparserModelEnum


class AnyparserOCRTool(AnyparserBaseTool):
    """Tool for extracting text from images using OCR."""

    name: str = "anyparser_ocr"
    description: str = """
    Extract text from images using OCR. Can handle:
    - PNG images
    - JPEG images
    - Scanned PDFs
    
    Supports multiple languages and OCR presets for better accuracy.
    """

    model: AnyparserModelEnum = Field(
        default=AnyparserModelEnum.OCR,
        description="Processing model (always OCR for this tool)",
    )
    file_path: Optional[List[str]] = Field(
        default=None,
        description="List of image file paths that this tool is restricted to process",
    )
    ocr_language: Optional[List[OcrLanguage]] = Field(
        default=None,
        description="Languages to use for OCR",
    )
    ocr_preset: Optional[OCRPreset] = Field(
        default=None,
        description="OCR preset configuration",
    )

    def __init__(self, **data):
        super().__init__(**data)
        if "Tool Description:" in self.description:
            self.description = self.description.split("Tool Description:")[-1].strip()

    def _run(self, *args) -> str:
        """Extract text from images using OCR.

        Args:
            args[0] (optional): List of file paths to process. Must match the initialized paths if file_path is set.

        Returns:
            The extracted text in the specified format.
        """
        try:
            # Handle both cases: with and without input
            if len(args) > 0:
                input_paths = args[0]
                # Validate paths if restricted
                if self.file_path is not None:
                    invalid_paths = [p for p in input_paths if p not in self.file_path]
                    if invalid_paths:
                        return f"Error: Can only process the following files: {self.file_path}"
                paths = input_paths
            else:
                if self.file_path is None:
                    return "Error: No file paths provided"
                paths = self.file_path

            # Initialize parser with OCR settings
            if not hasattr(self, "_parser"):
                options = self._get_parser_options(
                    ocr_language=self.ocr_language,
                    ocr_preset=self.ocr_preset,
                )
                self._parser = Anyparser(options=options)

            result = self._parse_sync(paths)

            if isinstance(result, str):
                return result

            return "\n".join([str(r) for r in result])

        except Exception as e:
            return f"Error processing images: {str(e)}"
